<?php
// Check for a form submission:
if ($_SERVER['REQUEST_METHOD'] == 'POST') {
	// Stores errors:
	$errors = array();

	// Need a payment token:
	if (isset($_POST['stripeToken'])) {

		$token = $_POST['stripeToken'];

		// Check for a duplicate submission, just in case:
		// Uses sessions, you could use a cookie instead.
		if (isset($_SESSION['token']) && ($_SESSION['token'] == $token)) {
			$errors['token'] = 'You have apparently resubmitted the form. Please do not do that.';
		} else { // New submission.
			$_SESSION['token'] = $token;
		}

	} else {
		$errors['token'] = 'You must enter payment information.';
	}

	// Set the order amount somehow:
	$amount = $totalCost ? $totalCost : $error['amount'] = 'No amount was specified.'; // $20, in cents
	
	// Validate other form data!

	// If no errors, process the order:
	if (empty($errors)) {

		// create the charge on Stripe's servers - this will charge the user's card
		try {

			// Include the Stripe library:
			// Assumes you've installed the Stripe PHP library using Composer!
			require_once SERVER_ROOT.'/addons/stripe/init.php';

			// set your secret key: remember to change this to your live secret key in production
			// see your keys here https://manage.stripe.com/account
			\Stripe\Stripe::setApiKey(($stripeLive ? STRIPE_PRIVATE_KEY : STRIPE_PRIVATE_KEY_TEST));

			// Charge the order:
			$charge = \Stripe\Charge::create
			(
				array 
				(
					"amount" => ($amount * 100), // amount in cents, again
					"currency" => "usd",
					"source" => $token,
					"description" => $description
				)
			);

			// Check that it was paid:
			if ($charge->paid == true) {

				// Store the order in the database.
				// Send the email.
				// Celebrate!

			} else { // Charge was not paid!
				$errors['card'] = '<strong>Payment System Error!</strong>Your payment could NOT be processed (i.e., you have not been charged) because the payment system rejected the transaction. You can try again or use another card.';
			}

		} catch (\Stripe\Error\Card $e) {
		    // Card was declined.
			$e_json = $e->getJsonBody();
			$err = $e_json['error'];
			$errors['stripe'] = $err['message'];
		} catch (\Stripe\Error\ApiConnection $e) {
		    // Network problem, perhaps try again.
			$e_json = $e->getJsonBody();
			$err = $e_json['error'];
			$errors['stripe'] = $err['message'];
		} catch (\Stripe\Error\InvalidRequest $e) {
		    // You screwed up in your programming. Shouldn't happen!
			$e_json = $e->getJsonBody();
			$err = $e_json['error'];
			$errors['stripe'] = $err['message'];
		} catch (\Stripe\Error\Api $e) {
		    // Stripe's servers are down!
			$e_json = $e->getJsonBody();
			$err = $e_json['error'];
			$errors['stripe'] = $err['message'];
		} catch (\Stripe\Error\Base $e) {
		    // Something else that's not the customer's fault.
			$e_json = $e->getJsonBody();
			$err = $e_json['error'];
			$errors['stripe'] = $err['message'];
		}

	} // A user form submission error occurred, handled below.

} // Form submission.
?>