<?php
/*created:20160831 modified:20171012*/

if(empty($Format))
{
	require_once SERVER_ROOT.'library/format.php';
	
	$GLOBALS['Format'] = new formatC();
}

class itemsC
{
	private $alias = NULL;
	private $discountApplied = array(); //all information pertaining to discount code entered
	private $discountsAutoApply = NULL; //list of all automatically applied discounts and their information; NULL is unitialized
	private $discountCode = NULL;
	
	private $discountAmount = NULL;
	private $discountItem = NULL;
	private $discountType = NULL;
	private $eligible = NULL; //holds purchased membership
	private $eritid = array();
	private $items = array(); //list of all items available for purchase and their information
	private $lastInsertedInvoiceId = NULL; //table id of the last insterted record
	private $otherItems = array(); //additional items not found in the items table
	private $purchases = array(); //list of all purchases and their information
	private $purchasedItems = array(); //list of purchased items containing their table id and quantity purchased
	private $quantity = array();
	private $tableName = NULL;
	
	public $discount = NULL; //total amount of discount
	public $erinid = NULL;
	public $insertValues = NULL;
	public $invoice = NULL;
	public $invoiceHTML = NULL;
	public $totalAmount = NULL; //total amount of all items purchased after applying the total discount
	public $totalCost = NULL; //total amount of all items purchased
	public $totalDiscount = NULL; //total amount of all applicable discounts
	public $totalDiscountDescription = NULL; //comma seperates string of all applied discount descriptions
	
	public function __construct($alias = NULL, $eritid = array(), $quantity = array(), $discountCode = NULL, $tableName = NULL)
	{
		$this->alias = $alias;		
		$this->eritid = $eritid;
		$this->quantity = $quantity;
		$this->tableName = $tableName ? $tableName : 'lmf_'.$alias;
		
		$this->setItems();
		$this->setPurchasedItems($eritid, $quantity);
		
		if($discountCode)
		{
			$this->discountCode = $discountCode;
			$this->setDiscountApplied($discountCode);
		}
		
		$this->calculateTotalCost();
	}
	
	public function addItem($item = array('amount' => NULL, 'quantity' => NULL, 'id' => NULL, 'item' => NULL, 'remaining' => NULL, 'specifics' => NULL))
	{
		/**
		* Add items not found in the items database table to items list
		* - Must have a unique id
		* - Id must be the same as what is used in the purchase section of the form
		*/
		
		if($item['id']) $this->items[$item['id']] = $item;
		
		$this->updatePurchaseItems($item['id']);
	}
	
	public function calculateDiscount($discount)
	{
		switch($discount['type'])
		{
			case 'item':
				if(!empty($this->purchasedItems[$discount['eritid']]['quantity']))
					return $discount['amount'] * $this->purchasedItems[$discount['eritid']]['quantity'];
			case 'percent':
				return ($this->getTotalCost * (100 - $discount['amount'])) / 100;
			case 'total':
				return $discount['amount'];
		}
	}
	
	public function calculateTotalCost() //this is needed to support old forms
	{
		global $mysql;
		
		$cost = array(); //stores all cost information
		$eritid = &$this->eritid;	
		$quantity = &$this->quantity;
		$quantityList = array(); //associate quantity with database table item id
		$totalCost = 0;
		
		//echo "SELECT * FROM `{$this->tableName}-items` ORDER BY `erit_id` ASC";
		$r = $mysql->query("SELECT * FROM `{$this->tableName}-items` ORDER BY `erit_id` ASC") or die($mysql->error.'line 47'); //get all available items
		
		if($r)
			while($row = $r->fetch_object())
				$cost[$row->erit_id] = array('amount' => $row->erit_amount, 'quantity' => $row->erit_quantity, 'id' => $row->erit_id, 'item' => $row->erit_item, 'remaining' => $row->erit_remaining, 'specifics' => $row->erit_specifics);
		
		//append additional costs
		$cost += $this->otherItems;
		
		for($i=0, $len=count($eritid); $i<$len; ++$i)
		{
			if(isset($cost[$eritid[$i]]))
			{
				$this->purchases[] = array('item' => $cost[$eritid[$i]]['item'], 'amount' => $cost[$eritid[$i]]['amount'], 'specifics' => $cost[$eritid[$i]]['specifics'], 'eritid' => $eritid[$i], 'quantity' => (int)$quantity[$i], 'subtotal' => $cost[$eritid[$i]]['amount'] * (int)$quantity[$i]);
				$totalCost += $quantity[$i] * $cost[$eritid[$i]]['amount'];
				$quantityList[$cost[$eritid[$i]]['id']] = $quantity[$i];
			}
		}
		
		//apply discount
		$totalDiscount = $this->getTotalDiscount();
		$totalCostBefore = $totalCost;
		
		if(isset($this->discountApplied['type']))
		{
			if($this->discountApplied['type'] == 'item' && !empty($quantityList[$this->discount['item']])) //item
			{
				$totalCost = $totalCost - ($this->discountApplied['amount'] * $quantityList[$this->discount['item']]);
				$this->discount = '$'.($this->discountApplied['amount'] * $quantityList[$this->discount['item']]);
			}
			
			if($this->discountApplied['type'] == 'percent') //percent
				$totalCost = ($totalCost * (100 - $this->discountApplied['amount'])) / 100;
			else if($this->discountApplied['type'] == 'total')
				$totalCost = $totalCost - $this->discountApplied['amount'];
		}
		
		$this->totalDiscount = $totalCostBefore - $totalCost;
		$this->totalCost = $this->totalAmount = $totalCost;
	}
	
	public function createInsertValues()
	{
		global $format;
		
		$insertValues = array();
		
		for($i=0, $len=count($this->purchases); $i<$len; ++$i)
			if($this->purchases[$i]['quantity'])
				$insertValues[] = '('.FormatC::databaseValue($this->purchases[$i]['amount']).','.FormatC::databaseValue($this->purchases[$i]['item']).','.FormatC::databaseValue($this->purchases[$i]['quantity']).','.FormatC::databaseValue($this->purchases[$i]['specifics']).','.FormatC::databaseValue($this->purchases[$i]['eritid']).','.FormatC::databaseValue($this->erinid).')';
		
		$this->insertValues = implode(',', $insertValues);
	}
	
	public function createInvoice()
	{
		$invoice = '';
		
		foreach($this->purchasedItems as $value)
			$invoice .= $value['item'].' - '.$value['specifics'].': $'.$value['amount'].' x '.$value['quantity'].' = $'.$value['amountTotal'].'
';
		
		if($this->totalDiscount)	
			$invoice .= 'Discount: '.$this->totalDiscountDescription.' - $'.$this->totalDiscount;
		
		$invoice .= '
		_____________________
		Total: $'.$this->totalAmount;
		
		$this->invoice = $invoice;
	}
	
	public function createInvoiceHTML()
	{
		$invoice = '';
		
		foreach($this->purchasedItems as $value)
			$invoice .= '<strong>'.$value['item'].' - '.$value['specifics'].' :</strong> $'.$value['amount'].' x '.$value['quantity'].' = $'.$value['amountTotal'].'<br />';
		
		if($this->totalDiscount)	
			$invoice .= '<strong>Discount:</strong> '.$this->totalDiscountDescription.' - $'.$this->totalDiscount.'<br />';
		
		$invoice .= '<div>_____________________</div><strong>Total :</strong> $'.$this->totalAmount;
		
		$this->invoiceHTML = $invoice;
	}
	
	public function eligibleForDiscount($elgibile)
	{
		if($elgibile === NULL || in_array($this->eligible, $elgibile) || (isset($_SESSION['portal']['user']['putid']) && in_array($_SESSION['portal']['user']['putid'], $elgibile)))
			return true;
		
		return false;
	}
	
	public function finalizeTransaction($amount, $description, $transactionId, $purchaserTableId)
	{
		/**
		* Performs all the necessary steps after payment has been confirmed and submission has inserted into the database
		* - Insert inovice information
		* - Insert purchase information
		* - Updates all items remaining
		*/
		
		$this->insertInvoiceRecord($amount, $description, $transactionId, $purchaserTableId);
		$this->insertPurchasesRecord();
		$this->updatePurchasedItemsRemainingRecord();
	}
	
	public function getDiscountsAutoApply()
	{
		if($this->discountsAutoApply === NULL)
			$this->setDiscountsAutoApply();
			
		return $this->discountsAutoApply;
	}
	
	public function getInvoice()
	{
		if(!$this->invoice)
			$this->createInvoice();
			
		return $this->invoice;
	}
	
	public function getInvoiceHTML()
	{
		if(!$this->invoiceHTML)
			$this->createInvoiceHTML();
			
		return $this->invoiceHTML;
	}
	
	public function getTotalAmount()
	{
		if($this->totalAmount === NULL)
			$this->setTotalAmount();
			
		return $this->totalAmount;
	}
	
	public function getTotalCost()
	{		
		if($this->totalCost === NULL)
			$this->setTotalCost();
		
		return $this->totalCost;
	}
	
	public function getTotalDiscount()
	{
		/**
		* - Get applicable automatically applied discounts
		* - Get applied discount
		*/
		
		if($this->totalDiscount === NULL)
			$this->setTotalDiscount();
			
		return $this->totalDiscount;
	}
	
	public function insertInvoiceRecord($amount, $description, $transactionId, $purchaserTableId)
	{
		global $Format, $mysql;
		
		//insert invoice
		$query = '
			`erin_amount` = '.FormatC::databaseValue($amount).',
			`erin_description` = '.FormatC::databaseValue($description).',
			`erin_date` = '.date('Ymd').',
			`erin_discount` = '.FormatC::databaseValue($this->getTotalDiscount()).',
			`erin_discountDescription` = '.FormatC::databaseValue($this->totalDiscountDescription).',
			`erin_time` = '.date('His').',
			`erin_transactionId` = '.FormatC::databaseValue($transactionId).',
			`TR_id` = '.$purchaserTableId;
				
		$mysql->query("INSERT `{$this->tableName}-invoices` SET $query") or die($mysql->error.' -- line 244: ');
		
		return ($this->lastInsertedInvoiceId = $mysql->insert_id);
	}
	
	public function insertPurchasesRecord()
	{
		/**
		* Inserts purchase information associated with last inserted invoice
		*/
		
		global $Format, $mysql;
		
		if($this->lastInsertedInvoiceId)
		{
			//build values
			$values = array();
			
			foreach($this->purchasedItems as $value)
				$values[] = 
				'('.
					FormatC::databaseValue($value['amount']).','.
					FormatC::databaseValue($value['item']).','.
					FormatC::databaseValue($value['quantity']).','.
					FormatC::databaseValue($value['specifics']).','.
					FormatC::databaseValue($value['eritid']).','.
					FormatC::databaseValue($this->lastInsertedInvoiceId).
				')';
			
			$mysql->query("INSERT `{$this->tableName}-purchases` (`erp_amount`, `erp_item`, `erp_quantity`, `erp_specifics`, `erit_id`, `erin_id`) VALUES ".implode(',', $values)) or die($mysql->error.' -- line 259: ');
		}
	}
	
	public function setDiscountApplied()
	{
		global $mysql;
	
		$r = $mysql->query("SELECT * FROM `{$this->tableName}-discounts` WHERE `erd_code` = '$this->discountCode' LIMIT 1");
		
		if($r && $row = $r->fetch_object())
			$this->discountApplied = $this->setDiscountInformation($row);
	}
	
	public function setDiscountInformation($row)
	{
		$discount = array(
		  'amount' => (float)$row->erd_amount,
		  'code' => $row->erd_code,
			'description' => NULL,
			'eligible' => NULL,
			'eritid' => (int)$row->erit_id,
			'type' => $row->erd_type
		);
		
		if(isset($row->erd_eligible))
			$discount['eligible'] = array_map(function($value)
			{
				return (int)$value;
			}, explode('|', $row->erd_eligible));
			
		if(isset($row->erd_description))
			$discount['description'] = $row->erd_description;
		
		return $discount;
	}
	
	public function setDiscountsAutoApply()
	{
		global $mysql;
		
		try
		{
			$r = $mysql->query("SELECT * FROM `{$this->tableName}-discounts` WHERE `erd_autoApply` = 1");// or die($mysql->error.' -- line 213');
			
			if($r)
				while($row = $r->fetch_object())
					$this->discountsAutoApply[] = $this->setDiscountInformation($row);
		}
		catch(Exception $e)
		{
		}
	}
	
	public function setEligible($eligible)
	{
		/**
		* Sets visitors eligibility
		* - $eligible can come from user input so sanitize
		*/
		
		global $Format;
		
		$this->eligible = FormatC::number($eligible);
	}
	
	public function setItemInformation($row)
	{
		return array
		(
		  'amount' => $row->erit_amount,
			'description' => $row->erit_description,
			'eritid' => $row->erit_id,
			'item' => $row->erit_item,
			'specifics' => $row->erit_specifics
		);
	}
	
	public function setItems()
	{
		/**
		* Gets all item informatin from the items database table
		*/
		
		global $mysql;
		
		$r = $mysql->query("SELECT * FROM `{$this->tableName}-items` ORDER BY `erit_id` ASC") or die($mysql->error.'line 280'); //get all available items
		
		if($r)
			while($row = $r->fetch_object())
				$this->items[$row->erit_id] = $this->setItemInformation($row);
	}
	
	public function setPurchasedItems($eritid, $quantity)
	{
		global $Format;
		
		for($i=0, $len=count($eritid); $i<$len; ++$i)
			if($quantity[$i])
				$this->setPurchaseItemInformation($eritid[$i], FormatC::number($quantity[$i]));
	}
	
	public function setPurchaseItemInformation($id, $quantity = NULL)
	{
		/**
		* Sets available purchase information
		* - Amount is the subtotal of the item's amount times the quantity
		*/
		
		$quantity = $quantity || $quantity === 0 ? $quantity : (isset($this->purchasedItems[$id]['quantity']) ? $this->purchasedItems[$id]['quantity'] : 0);
		$amount = isset($this->items[$id]['amount']) ? $this->items[$id]['amount'] * $quantity : NULL;
		
		$this->purchasedItems[$id] = array
		(
			'amount' => isset($this->items[$id]['amount']) ? $this->items[$id]['amount'] : NULL,
			'amountTotal' => $amount,
			'description' => (isset($this->items[$id]['description']) ? $this->items[$id]['description'] : NULL),
			'eritid' => (isset($this->items[$id]['eritid']) ? $this->items[$id]['eritid'] : NULL),
			'quantity' => $quantity,
			'item' => (isset($this->items[$id]['item']) ? $this->items[$id]['item'] : NULL),
			'specifics' => (isset($this->items[$id]['specifics']) ? $this->items[$id]['specifics'] : NULL),
		);
	}
	
	public function setSessionPurchases($token)
	{
		/**
		* This method sets the payment information for the given token
		*/
		
		if(isset($token, $_SESSION['forms'][$token]))
			$_SESSION['forms'][$token]['purchases'] = $this->purchases;
	}
	
	public function setTotalAmount()
	{
		$this->totalAmount = $this->getTotalCost() - $this->getTotalDiscount();
	}
	
	public function setTotalDiscount()
	{
		$this->totalDiscount = 0;
		$this->totalDiscountDescription = array();
		
		$discountsAutoApply = $this->getDiscountsAutoApply();
		
		for($i=0, $len=count($discountsAutoApply); $i<$len; ++$i)
			if($this->eligibleForDiscount($discountsAutoApply[$i]['eligible']))
			{
				$this->totalDiscount += $this->calculateDiscount($discountsAutoApply[$i]);		
				$discountsAutoApply[$i]['description'] ? $this->totalDiscountDescription[] = $discountsAutoApply[$i]['description'] : NULL;
			}
				
		if(isset($this->discountApplied['eligible'], $this->discountApplied['amount']) && $this->eligibleForDiscount($this->discountApplied['eligible']))
		{
			$this->totalDiscount += $this->calculateDiscount($this->discountApplied);
			$this->discountApplied['description'] ? $this->totalDiscountDescription[] = $this->discountApplied['description'] : NULL;
		}
		
		$this->totalDiscountDescription = implode(', ', $this->totalDiscountDescription);
	}
	
	public function setTotalCost()
	{
		$this->totalCost = 0;
		
		foreach($this->purchasedItems as $value)
			$this->totalCost += $value['amountTotal'];
	}
	
	public function updateItemsRemainingRecord($id, $quantity)
	{
		global $mysql;
		
		$mysql->query("UPDATE `{$this->tableName}-items` SET `erit_remaining` = (`erit_remaining` - $quantity) WHERE `erit_id` = $id") or die($mysql->error.' -- line 431');
	}
	
	public function updatePurchasedItemsRemainingRecord()
	{
		foreach($this->purchasedItems as $value)
			if($value['eritid'])
				$this->updateItemsRemainingRecord($value['eritid'], $value['quantity']);
	}
	
	public function updatePurchaseItems($id)
	{
		$this->setPurchaseItemInformation($id);
	}
	
	public function updateRemaining()
	{
		//keep for backwards compatability
		global $mysql;
		
		for($i=0, $len=count($this->purchases); $i<$len; ++$i)
			if($this->purchases[$i]['quantity'])
			{
				//echo "UPDATE `lmf_{$this->alias}-items` SET `erit_remaining` = (`erit_remaining` - {$this->purchases[$i]['quantity']}) WHERE `erit_id` = {$this->purchases[$i]['eritid']}";
				$mysql->query("UPDATE `{$this->tableName}-items` SET `erit_remaining` = (`erit_remaining` - {$this->purchases[$i]['quantity']}) WHERE `erit_id` = '{$this->purchases[$i]['eritid']}'") or die($mysql->error.' -- line 200');
			}
	}
}
?>