<?php

namespace Html;

/**
* This class html snippets for a consistant look and feel throughout the site.
*
* Created: 20171227 - Modified: 20180115
* @author Joshua P. Dale - littletzar.com
*

*/

abstract class HtmlC
{
  public $link = NULL;

  public function __construct()
  {
    $this->link = new Html\LinkC();
  }

  /**
  * This method builds a listTable.
  *
  * @param string layout - The structure to build
  * @param array properties - Stuff
  * @return string
  */

  public static function build($layout, $properties)
  {
    switch($layout)
    {
      case 'listTableItem':
        return Html\ListC::build('tableItem', $properties);
      case 'listTable':
        return Html\ListC::build('table', $properties);
      default:
        return '';
    }
  }

  /**
  * This method creates an html formatted message to send in an email.
  *
  * @param string message - The message
  * @return string
  */

  public static function createEmailMessage($message)
  {
    global $STYLES2;

    if(!isset($STYLES2))
    {
      ob_start();
      include SERVER_ROOT.'styles/site/site.css';
      include SERVER_ROOT.'styles/site/site-styles-configuration2.php';
      ob_get_clean();
    }

    return
'<html>'.
'<head>'.
'</head>'.
'<body>'.
  '<table bgcolor="'.$STYLES2['email']['header']['backgroundColor'].'" border="0" cellspacing="0" style="background-color:'.$STYLES2['email']['header']['backgroundColor'].'; border-bottom:2px solid '.$STYLES2['email']['header']['borderColor'].'; color:'.$STYLES2['email']['header']['color'].'" width="100%">'.
    '<tr>'.
      '<td></td>'.
      '<td style="padding:2.5%; text-align:left;" width="800px">'.
        '<img alt="'.SITE_NAME.'" src="'.SITE_URL.SITE_LOGO.'" height="'.$STYLES2['email']['header']['logoHeight'].'" title="'.SITE_NAME.'" width="auto" />'.
      '</td>'.
      '<td></td>'.
    '</tr>'.
  '</table>'.
  '<table bgcolor="'.$STYLES2['email']['body']['backgroundColor'].'" border="0" cellspacing="0" style="background-color:'.$STYLES2['email']['body']['backgroundColor'].';" width="100%">'.
    '<tr>'.
      '<td></td>'.
      '<td style="color:'.$STYLES2['email']['body']['color'].'; padding:2.5%; text-align:left;" width="800px">'.
        $message.
      '</td>'.
      '<td></td>'.
    '</tr>'.
  '</table>'.
  '<table bgcolor="'.$STYLES2['email']['footer']['backgroundColor'].'" border="0" cellspacing="0" style="background-color:'.$STYLES2['email']['footer']['backgroundColor'].'; border-top:2px solid '.$STYLES2['email']['header']['borderColor'].'; " width="100%">'.
    '<tr>'.
      '<td></td>'.
      '<td style="padding:2.5%; text-align:left;" width="800px">'.
        '<table width="100%">'.
          '<tr>'.
            '<td style="color:'.$STYLES2['email']['footer']['color'].'">'.
              SITE_NAME.'<br />'.
              (PHONE ? 'P <a href="tel:'.PHONE_LINK.'" style="color:'.$STYLES2['email']['footer']['color'].'">'.PHONE.'</a> ' : '').
              (PHONE_FREE ? 'P <a href="tel:'.PHONE_FREE_LINK.'" style="color:'.$STYLES2['email']['footer']['color'].'">'.PHONE_FREE.'</a>' : '').'<br />'.
              (ADDRESS ? '<a href="'.ADDRESS_LINK.'" style="color:'.$STYLES2['email']['footer']['color'].'" target="_blank">'.strip_tags(ADDRESS).'</a>' : '').
            '</td>'.
            ($STYLES2['email']['footer']['logo'] ?
            '<td style="text-align:right">'.
              '<img alt="'.SITE_NAME.'" src="'.$STYLES2['email']['footer']['logo'].'" height="55" title="'.SITE_NAME.'" width="auto" />'.
            '</td>' :
            '').
          '</tr>'.
        '</table>'.
      '</td>'.
      '<td></td>'.
    '</tr>'.
  '</table>'.
'</body>';
  }

  /**
  * This method creates the head title tag for this current page
  */

  public static function createPageTitle()
  {
    global $Page;

    $title = isset($Page) ? $Page->getHeading() : NULL;

    return '<title>'.($title ? $title.': ' : '').SITE_NAME.'</title>';
  }
  /**
  * This method creates the standard HTML table used throughout the site.
  *
  * @param array data - The data to add to the table
  * @param array headers - The table headers. $headers can be: List of dictionary data ['class' => 'className', 'text' => 'headerText'] | List of header text
  * @param array attributes = A dictionary of attributes to add to the table
  * @return string - The HTML formatted table
  */

  public static function createTable($data, $headers = [], $attributes = [])
  {
    $bodyText = '';
    $headerText = ''; //header text to put at the top of the table

    ob_start();

    if($data)
    {
      echo
        '<table',
          ' class="table'.(array_key_exists('class', $attributes) ? ' '.$attributes['class'] : ''),'"',
          (array_key_exists('id', $attributes) ? ' id="'.$attributes['id'].'"' : ''),
        '>';

      foreach($headers as $key => $value)
        $headerText .= '<th'.(isset($value['class']) ? ' class="'.$value['class'].'"' : '').'>'.(isset($value['text']) ? $value['text'] : (is_string($value) ? $value : NULL)).'</th>';

      if(trim(str_replace(array('<th>', '</th>'), '', $headerText)))
        echo '<tr>',$headerText,'</tr>';

      for($i=0, $len=count($data); $i<$len; ++$i)
        echo self::createTableRow($data[$i]);

      echo '</table>';
    }

    return ob_get_clean();
  }

  /**
  * This method creats a table row.
  *
  * @param array data - The data to create the row from
  * @return string|NULL - The table row
  */

  public static function createTableRow($data)
  {
    if(is_array($data))
    {
      ob_start();

      echo '<tr>';

      foreach($data as $value)
      {
        if(is_string($value))
          $value = array('text' => $value);

        echo
          '<td',
            (isset($value['class']) ? ' class="'.$value['class'].'"' : ''),
            (isset($value['id']) ? ' id="'.$value['id'].'"' : ''),
          '>'.$value['text'].'</td>';
      }

      echo '</tr>';

      return ob_get_clean();
    }

    return NULL;
  }

  /**
  * This method encodes an email address into it's ASCII character codes.
  *
  * @param string email - The email address to encode
  * @return string
  */

  public static function encodeEmail($email)
  {
    ob_start();

    for($i=0, $len=strlen($email); $i<$len; ++$i)
      echo '&#',ord($email[$i]),';';

    return ob_get_clean();
  }

  /**
  * This method converts a list of attributes into an Html valid string.
  *
  * @param array attributes - A list of attributes array('attribute' => 'value'))
  * @return string
  */

  public static function attributes2string($attributes)
  {
    ksort($attributes);

    ob_start();

    foreach($attributes as $key => $value)
      echo ' ',$key,'="',htmlentities($value),'"';

    return ob_get_clean();
  }
}
