var geocoder = {}; //stores geocoder class; only needs to be created once

var Map = function(canvas)
{
	/*use lat for latitude and lng for longitude because this is how google abbrviates it*/
	this.addresses = []; //a list of addresses that need at location
	this.bounds = new google.maps.LatLngBounds(); //map window zoom
	this.canvas = canvas;
	this.filterAlpha = 
	{
		alpha: true,
		end: null,
		start: null
	} //flags, start, and end for alpha filter
	this.filterArea = null;
	this.filterCaterogy = null;
	this.filterType = null;
	this.geocacheQueries = 0; //number of geocache calls; limit 10/s
	this.informationWindow = new google.maps.InfoWindow;
	this.latLongArray = []; //stores all latitude and longitude information for one map instance
	this.map = {}; //google map object
  this.mapCanvas = document.getElementById(canvas);
  //this.position = new google.maps.LatLng(38.0770059, -97.9170196);
  this.mapOptions = 
	{
    center: {lat:38.0770059, lng:-97.9170196},
    zoom: 15,
    mapTypeId: google.maps.MapTypeId.ROADMAP
  }
	this.marker =
	{
		color:'FE7569',
		colorDefault:'FE7569',
		image:this.getMarkerImage('FE7569'),
		imageDefault:this.getMarkerImage('FE7569')
	}
	this.places = [];

  this.map = new google.maps.Map(this.mapCanvas, this.mapOptions);
	this.wait = {}; //asynchonus functions needed to complete before performing certain actions
}

Map.prototype.addInformationWindow = function(marker, html) 
{	
  google.maps.event.addListener(marker, 'click', function() 
	{
		this.informationWindow.close();
		this.informationWindow.setContent(html);
		this.informationWindow.open(this.map, marker);
	}.bind(this));
}

Map.prototype.addMarker = function(latlong, args) 
{
	/*Adds a new map marker to map instance*/
	
  if(latlong) 
	{
				
    var options = 
		{
      animation: google.maps.Animation.DROP,
			icon:this.marker.image,
			opacity:.85,
      position: latlong,
      map: this.map
    };

    if (typeof args == 'object' && typeof args.name != 'undefined')
      options.title = args.name;

    var marker = new google.maps.Marker(options);
		
		var information = '<div class="mapInformationWindow">' +
			(typeof args.name != 'undefined' ? '<strong>' + args.name + '</strong>' : '') + 
			(typeof args.website != 'undefined' ? '<div><a href="' + args.website + '" target="_blank">Website</a></div>' : '') + 
			(typeof args.addressFull != 'undefined' ? '<p>' + args.addressFull + '</p>' : '') + 
			(typeof args.phone != 'undefined' || typeof args.phoneFree != 'undefined' ? '<p>' +
				(typeof args.phone != 'undefined' ? '<a href="tel:+1' + args.phone + '" style="display:block">' + args.phone + '</a>' : '') + 
				(typeof args.phoneFree != 'undefined' ? '<a href="tel:+1' + args.phoneFree + '" style="display:block">' + args.phoneFree + '</a>' : '') + '</p>' : '') + 
			(typeof args.hours != 'undefined' ? '<p>' + args.hours + '</p>' : '') + 
			(typeof args.moreInformation != 'undefined' ? '<p><a href="' + args.moreInformation + '">More Information</a></p>' : '') + '</div>';
			
		this.addInformationWindow(marker, information);
		
		return marker;
  }
};

Map.prototype.addPlace = function(args)
{
	if(typeof args == 'object')
	{
		if(typeof args.location == 'undefined' && typeof args.addressFull != 'undefined') //get location
		{
			/*tried writing a setTimeout function to handle more than 10 queries, but it would timeout, take too long, or do nothing*/
			
			if(this.geocacheQueries < 11) //ten query limit
				this.addresses.push(this.findLocation(args.addressFull, this.places.length, this)); //pass index to update places[] when done; pass this to remember class			;
		}
		else
			this.addresses.push(this.findLocation('', this.places.length, this)); //pass locations that don't need a location for processing with those that do
			
		this.places.push(args);
		
		return this.places.length - 1;
	}
}

Map.prototype.addAllPlacesToMap = function()
{
	this.addPlacesToMap(this.places)
}

Map.prototype.addPlacesToMap = function(places)
{
	/*
	* - Verify all places have a location (lat, lng)
	* -- If there is no location but there is an address, geocode the location from the address
	* - Add all markers to the map for all the places that have a location
	*/
	
	if(places.length)
	{
		//wait until all locations are found
		this.wait = $.when
		.apply(this, this.addresses)
		.done(function() 
		{
			/*
			- If only one place, will return a 1D array vs a 2D array
			*/
			
			var _this = {};
			//var bounds = new google.maps.LatLngBounds();
			
			typeof arguments == 'object' && typeof arguments[0] == 'object' && typeof arguments[0][0] == 'undefined' && (arguments[0] = Array.prototype.splice.call(arguments, 0)) && (arguments.length = 1); //remove and return all elements of the array and set lenght as 1; treating array like object as an array				
			_this = arguments[0][2]; //get this			
			_this.map.setCenter(_this.averageLatLong());
					
			places.map(function(value)
			{
				if(typeof value.location.lat != 'undefined' && typeof value.location.lng != 'undefined')
				{
					value.marker = _this.addMarker.call(_this, value.location, value);
					_this.bounds.extend(new google.maps.LatLng(value.location.lat, value.location.lng));
				}
			}, _this);
			
			_this.places.length > 1 && _this.map.fitBounds(_this.bounds);		
		})
		.fail(function()
		{
			console.log(arguments);
		});
	}
}

Map.prototype.averageLatLong = function()
{
	/*find the average center of all given points*/
	
  var lat = this.places.map(function(item) 
	{
  	return item.location.lat;
	}).reduce(function(total, current)
	{ 
		return total += current; 
	}) / this.places.length;
	
  var long = this.places.map(function(item) 
	{
  	return item.location.lng;
	}).reduce(function(total, current)
	{ 
		return total += current; 
	}) / this.places.length;
	
	return {lat:lat, lng:long};
}

Map.prototype.changeMarker = function(color, image)
{
	/*
	- image is not used right now; develop when needed
	*/
	
	this.marker = 
	{
		color:color,
		image:this.getMarkerImage(color)
	}
}

Map.prototype.changeMarkerColor = function(color)
{	
	this.marker = 
	{
		color:color,
		image:this.getMarkerImage(color)
	}
}

Map.prototype.filterByAlpha = function(filter)
{
	/*
	* - Expects:
	* -- Any character or character range
	* -- Single character (b)
	* -- Character ranger (a-f)
	* - Find start character
	* - Find end character
	* - If non alpha and a range: include all non alpha
	*/
	
	if(filter)
	{
		this.filterAlpha.start = filter[0];
		this.filterAlpha.end = filter[0] != filter[filter.length - 1] ? filter[filter.length - 1] : null;
		
		if(this.filterAlpha.start.toLowerCase().localeCompare('a') < 0 || this.filterAlpha.start.toLowerCase().localeCompare('z') > 0) //is non-alpha
			this.filterAlpha.alpha = false
		else
			this.filterAlpha.alpha = true
	}
	else
		this.filterAlpha.start = this.filterAlpha.end = null;
	
	this.filterPlaces();
}

Map.prototype.filterByArea = function(area)
{
	/*
	* - Expects: A single area string
	*/
	
	this.filterArea = area ? area : null;
	this.filterPlaces();
}

Map.prototype.filterByCategory = function(category)
{
	/*
	* - Expects: A single category string
	*/
	
	this.filterCategory = category ? category : null;
	this.filterPlaces();
}

Map.prototype.filterByType = function(type)
{
	/*
	* - Expects: A single type string
	*/
	
	this.filterType = type ? type : null;
	this.filterPlaces();
}

Map.prototype.filterPlaces = function()
{
	/*
	* - Itinerate through places
	* -- Compare first letter of name to alpha filter
	* -- Compare area to area filter
	* -- Compare categies to categories filter
	* -- Compare types to type filter
	* -- If all flags are true:
	* --- Show place on map
	* --- Show place on list if list is displayed
	* -- Else
	* --- Hide place on map
	* --- Hide place on list if list is displayed
	* - Reset flags
	*/

	var alpha = false; //is the place in the alpha range
	var area = false; //is the place in the selected area
	var category = false; //is the place in the selected category
	var type = false; //is the place the selected type
		
	for (var i=0, len=this.places.length; i<len; i++) 
	{
		if(this.filterAlpha.start == null) //show all
			alpha = true;
		else if(this.filterAlpha.alpha) //alpha and single character or character range
		{
			if(this.filterAlpha.end != null && this.places[i].name[0].toLowerCase().localeCompare(this.filterAlpha.start) >= 0 && this.places[i].name[0].toLowerCase().localeCompare(this.filterAlpha.end) <= 0) //in range
				alpha = true;
			else if(this.filterAlpha.end == null && this.places[i].name[0].toLowerCase().localeCompare(this.filterAlpha.start) == 0) //character match
				alpha = true;
		}
		else if(this.filterAlpha.end == null) //non-alpha and single character
		{
			if(this.places[i].name[0].toLowerCase().localeCompare(this.filterAlpha.start) == 0) //character match
				alpha = true;
		}
		else if(this.places[i].name[0].toLowerCase().localeCompare('a') < 0 || this.places[i].name[0].toLowerCase().localeCompare('z') > 0) //in range
			alpha = true;
			
		if(this.filterArea == null || this.places[i].areaId == this.filterArea)
			area = true;
			
		if(this.filterCategory == null || this.places[i].categories.indexOf(this.filterCategory) >= 0)
			category = true;
			
		if(this.filterType == null || this.places[i].typeId == this.filterType)
			type = true;
		
		if(alpha && area && category && type)
		{
			this.places[i].marker.setMap(this.map);
			var listing = document.getElementById('directoryContainer' + this.places[i].id);
			
			if(typeof listing != 'undefined')
				listing.style.display = '';
		}
		else
		{
			this.places[i].marker.setMap(null);
			var listing = document.getElementById('directoryContainer' + this.places[i].id);
			
			if(typeof listing != 'undefined')
				listing.style.display = 'none';
		}
		
		//reset flags		
		alpha = false;
		area = false;
		category = false;
		type = false;
	}
}

Map.prototype.findLocation = function(address, index, _this) 
{
	/*Get the latitude and longitude for an address*/	
  _this.geocacheQueries++;
  var deferred = $.Deferred();
	
	if(address == '')
		deferred.resolve({}, index, _this);
	else
	{		
		if(typeof geocoder.class == 'undefined')
			geocoder.class = new google.maps.Geocoder();
	
		geocoder.class.geocode(
		{
			'address': address
		},
		function(results, status) 
		{
			if(status == google.maps.GeocoderStatus.OK) 
			{
				if(typeof index != 'undefined' && typeof _this.places[index] != 'undefined') //add to places list
					_this.places[index].location = 
					{
						lat:results[0].geometry.location.lat(), 
						lng:results[0].geometry.location.lng()
					};
					
				deferred.resolve(results[0].geometry.location, index, _this);
			} 
			else
			{
				deferred.reject('Geocode was not successful for the following reason: ' + status);
			}
			
			_this.geocacheQueries--;		
		});
	}
	
  return deferred.promise();
};

Map.prototype.getMarkerImage = function(color, image, size, origin, anchor, scaledSize)
{
	/*
	- image, size, origin, anchor, and scaledSize are not used at this time; develop when needed later
	*/
	
	if(typeof color == 'undefined')
		color:this.marker.defaultColor;
			
	return {
		url:"http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld=%E2%80%A2|" + color,
    size:new google.maps.Size(24, 40),
    origin:new google.maps.Point(0, 0),
    anchor:new google.maps.Point(12, 40),
		scaledSize:new google.maps.Size(24, 40)
	};
}

Map.prototype.mapIt = function(vars, _this)
{
	if(typeof vars.placeIndex != 'undefined' && typeof _this.places[vars.placeIndex] != 'undefined')
		_this.map.setCenter(_this.places[vars.placeIndex].location);
	if(typeof vars.latitude != 'undefined' && typeof vars.longitude != 'undefined' && vars.latitude && vars.longitude)
		_this.map.setCenter({lat:vars.latitude, lng:vars.longitude});
	else if(typeof vars.addressFull != 'undefined')
		this.findLocation(args.addressFull, null, _this)
		
	google.maps.event.trigger(_this.places[vars.placeIndex].marker, 'click');
}