// Created by Larry Ullman, www.larryullman.com, @LarryUllman
// Posted as part of the series "Processing Payments with Stripe"
// http://www.larryullman.com/series/processing-payments-with-stripe/
// Last updated April 14, 2015

// This page is intended to be stored in a public "js" directory.

// This function is just used to display error messages on the page.
// Assumes there's an element with an ID of "payment-errors".
function reportError(msg, id) {
	// Show the error in the form:
	$(id).text(msg); //.addClass('alert alert-error');
	// re-enable the submit button:
	/*$('#submitBtn').prop('disabled', false);*/
	return false;
}

function clearError(id) {
	$(id).text('');
}

// Assumes jQuery is loaded!
// Watch for the document to be ready:
$(document).ready(function() {
	//disable forms ability to submit until Stripe returns a valid card
	//origional code: $('[type="submit"]').prop('disabled', true);
	
  //watch for all cc field completed and then test cc
	var card = [0,0,0,0]; //simple way to remember card information
	
	$('#cardName, #cardNumber, #cardCVC, #cardMonth, #cardYear').on('blur', function()
	{
		/**
		*- Check card after all information entered
		*- Remember card information for comparison if field is blurred after
		*-- If changed, get new token
		*/
		
		// Flag variable:
		var error = false, complete = 0, change = false;
		
		// Get the values:
		var ccNum = $('.card-number').val().replace(/[\D]/g, ''), cvcNum = $('.card-cvc').val().replace(/[\D]/g, ''), expMonth = $('.card-expiry-month').val().replace(/[\D]/g, ''), expYear = $('.card-expiry-year').val().replace(/[\D]/g, ''), name = document.getElementById('cardName').value;		

		// Validate name:
		if (name && name.replace(/[^a-zA-z]/g, '').length == 0) 
		{
			complete = false;
			error = true;
			reportError('The name on the card appears to be invalid.', '#cardNameError');
		}
		else if(name)
		{
			if(card[3] != name)
			{
				card[3] = name;
				change = true;
			}
				
			clearError('#cardNumberError');
			complete++;
		}	

		// Validate the number:
		if (ccNum && !Stripe.card.validateCardNumber(ccNum)) 
		{
			complete = false;
			error = true;
			reportError('The credit card number appears to be invalid.', '#cardNumberError');
		}
		else if(ccNum)
		{
			if(card[0] != ccNum)
			{
				card[0] = ccNum;
				change = true;
			}
				
			clearError('#cardNumberError');
			complete++;
		}

		// Validate the CVC:
		if (cvcNum && !Stripe.card.validateCVC(cvcNum)) 
		{
			complete = false;
			error = true;
			reportError('The CVC number appears to be invalid.', '#cardCVCError');
		}
		else if(cvcNum)
		{
			if(card[1] != cvcNum)
			{
				card[1] = cvcNum;
				change = true;
			}
				
			clearError('#cardCVCError');
			complete++;
		}

		// Validate the expiration:
		if (expMonth && expYear && !Stripe.card.validateExpiry(expMonth, expYear)) {
			complete = false;
			error = true;
			reportError('The expiration date appears to be invalid.', '#cardExpError');
		}
		else if(expMonth && expYear)
		{
			if(card[2] != (expMonth + '/' + expYear))
			{
				card[2] = expMonth + '/' + expYear;
				change = true;
			}
				
			clearError('#cardExpError');
			complete++;
		}

		// Check for errors:
		if (change && complete == 4 && !error) {
console.log('complete');
			// Get the Stripe token:
			Stripe.card.createToken({
				name: name,
				number: ccNum,
				cvc: cvcNum,
				exp_month: expMonth,
				exp_year: expYear
			}, stripeResponseHandler);

		}
	});
	
	/*origional way
	// Watch for a form submission:
	$("#payment-form").submit(function(event) {

		// Flag variable:
		var error = false;

		// disable the submit button to prevent repeated clicks:
		$('#submitBtn').attr("disabled", "disabled");

		// Get the values:
		var ccNum = $('.card-number').val(), cvcNum = $('.card-cvc').val(), expMonth = $('.card-expiry-month').val(), expYear = $('.card-expiry-year').val();

		// Validate the number:
		if (!Stripe.card.validateCardNumber(ccNum)) {
			error = true;
			reportError('The credit card number appears to be invalid.');
		}

		// Validate the CVC:
		if (!Stripe.card.validateCVC(cvcNum)) {
			error = true;
			reportError('The CVC number appears to be invalid.');
		}

		// Validate the expiration:
		if (!Stripe.card.validateExpiry(expMonth, expYear)) {
			error = true;
			reportError('The expiration date appears to be invalid.');
		}

		// Validate other form elements, if needed!

		// Check for errors:
		if (!error) {

			// Get the Stripe token:
			Stripe.card.createToken({
				number: ccNum,
				cvc: cvcNum,
				exp_month: expMonth,
				exp_year: expYear
			}, stripeResponseHandler);

		}

		// Prevent the form from submitting:
		return false;

	}); // Form submission*/

}); // Document ready.

// Function handles the Stripe response:
function stripeResponseHandler(status, response) {

	// Check for an error:
	if (response.error) {

		reportError(response.error.message, '#stripeToken');
		purchaseTotal.submitEnableToggle('payment', false) //new code

	} else { // No errors, submit the form:
    $('#stripeToken').html('<input type="hidden" name="stripeToken" value="' + response['id'] + '" />');		
		//origional code: $('[type="submit"]').prop('disabled', false);
		purchaseTotal.submitEnableToggle('payment', true) //new code
		
	  /*origional way of creating token
		var f = $("#payment-form");

	  // Token contains id, last4, and card type:
	  var token = response['id'];

	  // Insert the token into the form so it gets submitted to the server
	  f.append("<input type='hidden' name='stripeToken' value='" + token + "' />");

	  // Submit the form:
	  f.get(0).submit();*/

	}

} // End of stripeResponseHandler() function.