<?php
/************************************************************************************************
* Copyright (C)2011 - 2021 littletzar - All Rights Reserved                                     *
* Unauthorized reproduction of this File in whole or part via any medium is strictly prohibited *
* Proprietary and confidential                                                                  *
* Written by Joshua P. Dale <littletzar@littletzar.com> - littletzar.com                        *
* For use by permission only                                                                    *
************************************************************************************************/

//initialize CMS
$topDirectory = __DIR__;
require_once rtrim($_SERVER['DOCUMENT_ROOT'], '/').'/load-conn.php';

/**
* This class manages styling.
*
* @created  20210421
* @modified 20210421
* @author   Joshua P. Dale - littletzar.com
*/

abstract class StylesC extends \ClassC
{
  public static $background = array
  (
    'gradient' => array
    (
      'lightBlueLeft' =>
      	'background:#00adef;'. //lightBlue
        'background:-moz-linear-gradient(left, rgba(0,173,239,1) 0%, rgba(0,173,239,1) 30%, rgba(0,173,239,.95) 45%, rgba(125,185,232,0) 100%);'.
        'background:-webkit-linear-gradient(left, rgba(0,173,239,1) 0%, rgba(0,173,239,1) 30%, rgba(0,173,239,.95) 45%, rgba(125,185,232,0) 100%);'.
        'background:linear-gradient(left, rgba(0,173,239,1) 0%, rgba(0,173,239,1) 30%, rgba(0,173,239,.95) 45%, rgba(125,185,232,0) 100%);'.
        'filter:progid:DXImageTransform.Microsoft.gradient(startColorstr=\'#00adef\', endColorstr=\'#007db9e8\',GradientType=1);',
      'lightBlueLeft2darkBlueRight' =>
        //Permalink - use to edit and share this gradient: http://colorzilla.com/gradient-editor/#00adef+0,013e7f+100
        'background: rgb(0,173,239);'.
        'background: -moz-linear-gradient(left, rgba(0,173,239,1) 0%, rgba(1,62,127,1) 100%);'.
        'background: -webkit-linear-gradient(left, rgba(0,173,239,1) 0%,rgba(1,62,127,1) 100%);'.
        'background: linear-gradient(to right, rgba(0,173,239,1) 0%,rgba(1,62,127,1) 100%);'.
        'filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=\'#00adef\', endColorstr=\'#013e7f\',GradientType=1 );'
    )
  );
  public static $boxshadow =
  [
    'dark' => 'box-shadow: 0 5px 15px 1px rgba(0,0,0,.4)', //think about making more like this where it can only be applied to one property
    'primary' =>  '0 5px 15px 1px #cccccc'
  ];
  /*
  'border' => array
  (
    'image' => array
    (
      'red' => SITE_ROOT.'images/border-red.gif'
    )
  ), */
  public static $contentWidth = '1366px'; /*https://gs.statcounter.com/screen-resolution-stats*/
  public static $color =
  [
    'ashGray' => 'hsla(76, 15%, 76%, 1)',
    'black' => 'hsla(0, 0, 14%, 1)',
    'prussianBlue' => 'hsla(209, 88%, 16%, 1)',
    'shadowBlue' => 'hsla(215, 19%, 55%, 1)',
    'yellow' => 'hsla(50, 84%, 69%, 1)',
  ];
  public static $font =
  [
    'color' =>
    [
      'primary' => '#5f6b78',
      'secondary' => '#013e7f',
      'tertiary' => '#00adef',
      'faded' => '#acacac',
    ],
    'family' =>
    [
      'alternative' => '\'Open Sans Condensed\', sans-serif',
      'heading'     => '\'Nunito\', sans-serif',
      'main'        => '\'Nunito\', sans-serif'
    ],
    'shadow' =>
    [
      'dark'  => '1px 1px 1px rgba(0,0,0,.65)',
      'light' => '1px 1px 1px rgba(255,255,255,.65)'
    ]
  ];
  /*
  'margin' => function($value)
  {
    if($value = \FormatC::number($value, true))
      return (1 * $value).'rem'; //number_format(pow(1,($value * 10) - 10), 3).'rem';

    return '10rem';
  },
  'padding' => function($value)
  {
    if($value = \FormatC::number($value, true))
      return (1 * $value).'rem'; //number_format(pow(1,($value * 10) - 10), 3).'rem';

    return '10rem';
  },
  'snippets' => array
  (
    'contentBlock' =>
    '
      margin:0 auto;
      max-width:1024px;
      padding:0 2rem;
      position:relative;
      width:calc(100% - 4rem)
    '
  )*/

  /**
  * This method proportionally sets the font size.
  *
  * @param float $value - The scaled size from base
  * @return string - Returns scaled font size in 'rem'.  Defaults to '10rem' to show there is a problem.
  */

  public static function fontSize($value) //offsets it by 1.1 times
  {
    if($value = \FormatC::number($value, true))
      return number_format(pow(1.1,($value * 10) - 10), 3).'rem';

    return '10rem';
  }

  public static function margin($value) //offsets it by 1.1 times
  {
    if($value = \FormatC::number($value, true))
      return number_format(pow(1.1,($value * 10) - 10), 3).'rem';

    return '10rem';
  }

  public static function padding($value) //offsets it by 1.1 times
  {
    if($value = \FormatC::number($value, true))
      return number_format(pow(1.1,($value * 10) - 10), 3).'rem';

    return '10rem';
  }
}

/*

$STYLE_BLOCKS = array
(
  //properties
  'borderTopLightBlue' => 'border-top:5px solid #00adef',
  'fontFamilyPrimary' => 'font-family:'.$STYLES['font']['family']['primary'],
  'fontFamilySecondary' => 'font-family:'.$STYLES['font']['family']['secondary'],

  //methods
  position => function($left, $top)
  {
    switch($left)
    {
      case 'center':
        $left = '50%';
        break;
      case 'left':
        $left = '0';
        break;
      case 'right':
        $left = '100%';
        break;
    }

    switch($top)
    {
      case 'bottom':
        $top = '100%';
        break;
      case 'middle':
        $top = '50%';
        break;
      case 'top':
        $top = '0';
        break;
    }

    return
'position:absolute;'.
'left:'.$left.';'.
'top:'.$top.';'.
'transform:translate(-'.$left.', -'.$top.')';
  }
);*/
